// Generated by CoffeeScript 2.7.0
(function() {
  var AmqpClient, EventEmitter, ExchangeWrapper, QueueWrapper, amqp, log, wait,
    boundMethodCheck = function(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new Error('Bound instance method accessed before binding'); } };

  log = require('loglevel').getLogger('amqp');

  amqp = require('amqplib');

  ({EventEmitter} = require('events'));

  ExchangeWrapper = require('./exchange-wrapper');

  QueueWrapper = require('./queue-wrapper');

  wait = function(time) {
    return new Promise(function(rs) {
      return setTimeout(rs, time);
    });
  };

  module.exports = AmqpClient = class AmqpClient extends EventEmitter {
    constructor(conf, compat = require('./compat-node-amqp')) {
      var ref;
      super();
      this.connect = this.connect.bind(this);
      this.getChannel = this.getChannel.bind(this);
      this._exchange = this._exchange.bind(this);
      this.exchange = this.exchange.bind(this);
      this._queue = this._queue.bind(this);
      this.queue = this.queue.bind(this);
      this._bind = this._bind.bind(this);
      this.bind = this.bind.bind(this);
      this._unbind = this._unbind.bind(this);
      this.unbind = this.unbind.bind(this);
      this.shutdown = this.shutdown.bind(this);
      this.conf = conf;
      this.compat = compat;
      this.exchanges = {};
      this.queues = {};
      // subscribe the error handler, if specified
      if (typeof ((ref = this.conf) != null ? ref.errorHandler : void 0) === 'function') {
        this.on('error', this.conf.errorHandler);
      }
      // attach a fallback error listener. if there are no other
      // listeners attached, we will throw an error and possibly
      // crash the application if there are no errors attached to the
      // process.
      this.on('error', (err) => {
        if (this.listenerCount('error') === 1) {
          throw err;
        }
      });
    }

    connect() {
      var opts, uri, url;
      boundMethodCheck(this, AmqpClient);
      [uri, opts] = this.compat.connection(this.conf);
      url = new URL(uri);
      url.password = url.password.replace(/./g, '*');
      log.info("connecting to:", url.toString());
      return amqp.connect(uri, opts).then((conn) => {
        this.conn = conn;
        log.info("connected");
        // create a channel for sending AMQP commands
        this.channel = this.getChannel();
        // emit an error on connection 'close' event, unless we are
        // shutting down
        this.conn.on('close', (e) => {
          if (!this.shuttingDown) {
            return this.emit('error', e);
          }
        });
        return Promise.resolve("ok");
      }).catch((err) => {
        var time;
        if (this.conf.waitForConnection) {
          time = this.conf.waitForConnection;
          if (typeof time !== 'number') {
            time = 1000;
          }
          log.info("waiting for connection to:", uri);
          return wait(time).then(() => {
            return this.connect(uri, opts);
          });
        } else {
          return Promise.reject(err);
        }
      });
    }

    getChannel() {
      boundMethodCheck(this, AmqpClient);
      return this.conn.createChannel().then((c) => {
        // set max listeners to something arbitrarily large in
        // order to avoid misleading 'memory leak' error messages
        c.setMaxListeners(this.conf.maxListeners || 1000);
        // emit an error on channel 'close' event, unless we are
        // shutting down
        c.on('close', () => {
          if (!this.shuttingDown) {
            return this.emit('error', new Error('Channel closed'));
          }
        });
        return Promise.resolve(c);
      }).catch(function(err) {
        return Promise.reject(err);
      });
    }

    _exchange(name, type, opts) {
      boundMethodCheck(this, AmqpClient);
      if (name instanceof ExchangeWrapper) {
        return name;
      }
      if (this.exchanges[name]) {
        return this.exchanges[name];
      }
      return this.channel.then((c) => {
        return (!type ? name === '' ? Promise.resolve({
          exchange: ''
        }) : c.checkExchange(name).then(function() {
          return {
            exchange: name
          };
        }) : c.assertExchange(name, type, opts)).then((e) => {
          log.info('exchange ready:', e.exchange);
          return this.exchanges[name] = new ExchangeWrapper(this, e, c);
        });
      }).catch(function(err) {
        log.error("_exchange error", err);
        return Promise.reject(err);
      });
    }

    exchange() {
      var name, opts, type;
      boundMethodCheck(this, AmqpClient);
      [name, type, opts] = this.compat.exchangeArgs(...arguments);
      return this.compat.promise(this._exchange(name, type, opts));
    }

    _queue(qname, opts) {
      boundMethodCheck(this, AmqpClient);
      if (qname instanceof QueueWrapper) {
        return Promise.resolve(qname);
      }
      if (qname !== null && typeof qname === 'object') {
        opts = qname;
        qname = '';
      }
      if (!qname) {
        qname = '';
      }
      opts = opts != null ? opts : qname === '' ? {
        exclusive: true
      } : void 0;
      if (this.queues[qname] && qname !== '') {
        return this.queues[qname];
      }
      return this.channel.then((c) => {
        return (!opts ? c.checkQueue(qname) : c.assertQueue(qname, opts)).then((q) => {
          log.info('queue created:', q.queue);
          return this.queues[qname] = new QueueWrapper(this, q, c);
        });
      }).catch(function(err) {
        log.error("_queue error", err);
        return Promise.reject(err);
      });
    }

    queue() {
      var opts, qname;
      boundMethodCheck(this, AmqpClient);
      [qname, opts] = this.compat.queueArgs.apply(void 0, arguments);
      return this.compat.promise(this._queue(qname, opts));
    }

    _bind(exchange, queue, topic, callback) {
      boundMethodCheck(this, AmqpClient);
      if (typeof topic === 'function') {
        callback = topic;
        topic = queue;
        queue = '';
      }
      return (Promise.all([this._exchange(exchange), this._queue(queue)])).then(function([ex, q]) {
        return q.bind(ex, topic).then(function(q) {
          if (callback != null) {
            return q.subscribe(callback);
          }
        }).then(function() {
          return q.name;
        });
      }).catch(function(err) {
        log.error("_bind error", err);
        return Promise.reject(err);
      });
    }

    bind(exchange, queue, topic, callback) {
      boundMethodCheck(this, AmqpClient);
      return this.compat.promise(this._bind(exchange, queue, topic, callback));
    }

    _unbind(queue) {
      boundMethodCheck(this, AmqpClient);
      return this.queue(queue).then(function(q) {
        return q.unbind();
      }).then(function(q) {
        return q.unsubscribe();
      }).then(function() {
        return queue;
      });
    }

    unbind(queue) {
      boundMethodCheck(this, AmqpClient);
      return this.compat.promise(this._unbind(queue));
    }

    shutdown() {
      var ref;
      boundMethodCheck(this, AmqpClient);
      this.shuttingDown = true;
      return this.compat.promise((ref = this.conn) != null ? ref.close() : void 0);
    }

  };

}).call(this);
