// Generated by CoffeeScript 2.7.0
(function() {
  var err, log, promise;

  log = require('loglevel').getLogger('amqp');

  promise = void 0;

  try {
    promise = require('q');
    log.info('Q promises available');
  } catch (error) {
    err = error;
    promise = function(v) {
      return Promise.resolve(v);
    };
  }

  module.exports = {
    promise: promise,
    connection: function(conf) {
      var conn;
      conn = conf.connection;
      if (conn.url) {
        return [conn.url];
      } else {
        return [`amqp://${conn.login}:${conn.password}@${conn.host}/${conn.vhost}`];
      }
    },
    exchangeArgs: function(name, opts) {
      var k, o, type, v;
      if (Object.keys(opts != null ? opts : {}).length !== 0 && !(opts != null ? opts.passive : void 0)) {
        type = opts.type || 'topic';
        o = {};
        for (k in opts) {
          v = opts[k];
          if ((k === 'durable' || k === 'autoDelete') && v) {
            o[k] = v;
          }
        }
        if (o.durable == null) {
          o.durable = false;
        }
        return [name, type, o];
      } else {
        return [name];
      }
    },
    queueArgs: function(qname, opts) {
      return [qname, opts];
    },
    publishArgs: function(routingKey, message, options) {
      if (!options.contentType) {
        options.contentType = 'application/octet-stream';
      }
      if (typeof message === 'string') {
        message = Buffer.from(message);
      }
      if (typeof message === 'object' && !(message instanceof Buffer)) {
        message = Buffer.from(JSON.stringify(message));
        options.contentType = 'application/json';
      }
      return [routingKey, message, options];
    },
    subscribeOpts: function(opts) {
      return {
        noAck: !(opts != null ? opts.ack : void 0),
        exclusive: opts != null ? opts.exclusive : void 0,
        prefetch: (opts != null ? opts.prefetchCount : void 0) != null ? opts.prefetchCount : 1
      };
    },
    callback: function(channel, cb) {
      return function(data) {
        var ack, content, headers, info, ref, ref1, ref10, ref11, ref12, ref13, ref14, ref15, ref16, ref17, ref18, ref19, ref2, ref20, ref3, ref4, ref5, ref6, ref7, ref8, ref9;
        headers = ((ref = data.properties) != null ? ref.headers : void 0) || {};
        info = {
          consumerTag: (ref1 = data.fields) != null ? ref1.consumerTag : void 0,
          deliveryTag: (ref2 = data.fields) != null ? ref2.deliveryTag : void 0,
          redelivered: (ref3 = data.fields) != null ? ref3.redelivered : void 0,
          exchange: (ref4 = data.fields) != null ? ref4.exchange : void 0,
          routingKey: (ref5 = data.fields) != null ? ref5.routingKey : void 0,
          contentType: (ref6 = data.properties) != null ? ref6.contentType : void 0,
          contentEncoding: (ref7 = data.properties) != null ? ref7.contentEncoding : void 0,
          headers: (ref8 = data.properties) != null ? ref8.headers : void 0,
          deliveryMode: (ref9 = data.properties) != null ? ref9.deliveryMode : void 0,
          priority: (ref10 = data.properties) != null ? ref10.priority : void 0,
          correlationId: (ref11 = data.properties) != null ? ref11.correlationId : void 0,
          replyTo: (ref12 = data.properties) != null ? ref12.replyTo : void 0,
          expiration: (ref13 = data.properties) != null ? ref13.expiration : void 0,
          messageId: (ref14 = data.properties) != null ? ref14.messageId : void 0,
          timestamp: (ref15 = data.properties) != null ? ref15.timestamp : void 0,
          type: (ref16 = data.properties) != null ? ref16.type : void 0,
          userId: (ref17 = data.properties) != null ? ref17.userId : void 0,
          appId: (ref18 = data.properties) != null ? ref18.appId : void 0,
          clusterId: (ref19 = data.properties) != null ? ref19.clusterId : void 0
        };
        content = (ref20 = info.contentType) === 'application/json' || ref20 === 'text/json' ? JSON.parse(data.content) : {
          data: data.content,
          contentType: info.contentType
        };
        ack = {
          acknowledge: function() {
            return channel.ack(data);
          }
        };
        return cb(content, headers, info, ack);
      };
    }
  };

}).call(this);
