// Generated by CoffeeScript 2.7.0
(function() {
  var QueueWrapper, log;

  log = require('loglevel').getLogger('amqp');

  module.exports = QueueWrapper = class QueueWrapper {
    constructor(client, queue, channel) {
      this._bind = this._bind.bind(this);
      this.bind = this.bind.bind(this);
      this._unbind = this._unbind.bind(this);
      this.unbind = this.unbind.bind(this);
      this._subscribe = this._subscribe.bind(this);
      this.subscribe = this.subscribe.bind(this);
      this._unsubscribe = this._unsubscribe.bind(this);
      this.unsubscribe = this.unsubscribe.bind(this);
      this.isDurable = this.isDurable.bind(this);
      this.isAutoDelete = this.isAutoDelete.bind(this);
      this.client = client;
      this.queue = queue;
      this.channel = channel;
      this.name = this.queue.queue;
    }

    _bind(exchange, topic) {
      if (!topic) {
        topic = '';
      }
      return Promise.resolve().then(() => {
        if (typeof topic !== 'string') {
          throw new Error('Topic is not a string');
        }
        return Promise.all([this.unbind(), this.client.exchange(exchange)]);
      }).then(([whatevs, exchange]) => {
        log.info('binding:', exchange.name, this.name, topic);
        return this.channel.bindQueue(this.name, exchange.name, topic).then(() => {
          this._exchange = exchange.name;
          this._topic = topic;
          log.info('queue bound:', this.name, this._topic);
          return this;
        });
      }).catch(function(err) {
        log.error("qw _bind error", err);
        return Promise.reject(err);
      });
    }

    bind(exchange, topic) {
      return this.client.compat.promise(this._bind(exchange, topic));
    }

    _unbind() {
      if (!(this._exchange && this._topic)) {
        return Promise.resolve(this);
      }
      return this.channel.unbindQueue(this.name, this._exchange, this._topic).then(() => {
        log.info('queue unbound:', this.name, this._topic);
        delete this._exchange;
        delete this._topic;
        return this;
      }).catch(function(err) {
        log.error("qw _unbind error", err);
        return Promise.reject(err);
      });
    }

    unbind() {
      return this.client.compat.promise(this._unbind());
    }

    _subscribe(opts, callback) {
      if (typeof opts === 'function') {
        callback = opts;
        opts = null;
      }
      return this.unsubscribe().then(() => {
        if (this.consumerChannel) {
          return this.consumerChannel;
        } else {
          return this.client.getChannel();
        }
      }).then((consumerChannel) => {
        this.consumerChannel = consumerChannel;
        opts = this.client.compat.subscribeOpts(opts);
        // this is questionable and open to race conditions,
        // but there is no other way to do it
        return Promise.all([this.consumerChannel.prefetch(opts.prefetch), this.consumerChannel.consume(this.queue.queue, this.client.compat.callback(this.consumerChannel, callback), opts)]).then(([whatevs, {consumerTag}]) => {
          this._consumerTag = consumerTag;
          log.info('subscribed:', this.name, this._consumerTag);
          return this;
        });
      }).catch(function(err) {
        log.error("qw _subscribe error", err);
        return Promise.reject(err);
      });
    }

    subscribe(opts, callback) {
      return this.client.compat.promise(this._subscribe(opts, callback));
    }

    _unsubscribe() {
      if (!(this._consumerTag && this.consumerChannel)) {
        return this;
      }
      return this.consumerChannel.cancel(this._consumerTag).then(() => {
        log.info('unsubscribed:', this.name, this._consumerTag);
        delete this._consumerTag;
        return this;
      }).catch(function(err) {
        return log.error("qw _unsub error", err);
      });
    }

    unsubscribe() {
      return this.client.compat.promise(this._unsubscribe());
    }

    isDurable() {
      return this.queue.options.durable;
    }

    isAutoDelete() {
      return this.queue.options.autoDelete;
    }

  };

}).call(this);
