// Generated by CoffeeScript 2.7.0
(function() {
  var RpcBackend, compress, decompress, doack, log, merge;

  log = require('loglevel').getLogger('amqp');

  merge = require('./merge');

  ({compress, decompress} = require('./compressor'));

  // only ack if we actually are in ack mode, otherwise business as usual
  // to be backwards compatible.
  doack = function(opts, ack) {
    return function(cb) {
      if (opts != null ? opts.ack : void 0) {
        return Promise.resolve().then(function() {
          return cb();
        }).then(function() {
          return ack.acknowledge(false); // accept
        }).catch(function() {
          return ack.acknowledge(true, false); // reject, no requeue
        });
      } else {
        // deliberately not rethrow
        return cb();
      }
    };
  };

  module.exports = RpcBackend = class RpcBackend {
    constructor(client) {
      this.serve = this.serve.bind(this);
      this.client = client;
    }

    serve(exname, topic, opts, callback) {
      // backwards compatible with 3 args
      if (typeof opts === 'function') {
        callback = opts;
        opts = null;
      }
      opts = opts != null ? opts : {};
      return Promise.all([
        this.client.exchange(exname,
        {
          type: 'topic',
          durable: true,
          autoDelete: false
        }),
        this.client.exchange(''),
        this.client.queue(`${exname}.${topic}`,
        {
          durable: true,
          autoDelete: false
        })
      ]).then(([ex, defaultex, queue]) => {
        queue.bind(ex, topic);
        return queue.subscribe(opts, this._mkcallback(defaultex, callback, opts));
      });
    }

    // Creates a callback funtion which respects replyTo/correlationId
    _mkcallback(exchange, handler, subopts, serializeError = require('./error-serializer')) {
      return function(msg, headers, info, ack) {
        return doack(subopts, ack)(function() {
          var ct, opts, p, ref, timeout, timestamp;
          // no replyTo, no rpc
          if (info.replyTo == null) {
            return;
          }
          // in amqp info is bogus since it got seconds as resolution
          // we use timestamp in headers and expect milliseconds and
          // fall back on amqp default field.
          timestamp = (ref = headers != null ? headers.timestamp : void 0) != null ? ref : (info != null ? info.timestamp : void 0) * 1000;
          timeout = headers != null ? headers.timeout : void 0;
          // if we have both a timestamp and timeout we can discard
          // the message if it has expired timestamp
          if (timestamp && timeout) {
            if (typeof timestamp === 'string') {
              timestamp = new Date(timestamp).getTime();
            }
            // discard if timeout is passed
            if (Date.now() > timestamp + timeout) {
              log.info(`Discarding timed out message (${info.replyTo}, ${info.correlationId})`);
              return;
            }
          }
          // these options are used when publishing the result and/or
          // error messages
          opts = {};
          if (info.correlationId != null) {
            opts.correlationId = info.correlationId;
          }
          // maybe decompress compressed payload
          [ct, p] = decompress(msg, headers);
          merge(info, ct);
          return p.then(function(payload) {
            return Promise.resolve(handler(payload, headers, info));
          }).then(function(res) {
            var h;
            [h, p] = compress(res, headers);
            if (h) {
              opts.headers = h;
            }
            return p;
          }).then(function(res) {
            // then send the result
            return exchange.publish(info.replyTo, res, opts);
          }).catch(function(err) {
            log.error(err);
            return Promise.resolve().then(function() {
              return exchange.publish(info.replyTo, serializeError(err), opts);
            }).then(function() {
              if (subopts != null ? subopts.ack : void 0) { // let ack see error
                throw err;
              }
            }).catch(function(perr) {
              if (subopts != null ? subopts.ack : void 0) { // let ack see publish error
                throw perr;
              }
            });
          });
        });
      };
    }

  };

}).call(this);
