// Generated by CoffeeScript 2.7.0
(function() {
  var Cache, DEFAULT_TIMEOUT, Rpc, compress, decompress, merge, uuid;

  uuid = require('uuid');

  Cache = require('mem-cache');

  merge = require('./merge');

  ({compress, decompress} = require('./compressor'));

  DEFAULT_TIMEOUT = 1000;

  module.exports = Rpc = class Rpc {
    constructor(client, options1) {
      var ref;
      this.returnChannel = this.returnChannel.bind(this);
      this.registerResponse = this.registerResponse.bind(this);
      this.resolveResponse = this.resolveResponse.bind(this);
      this._rpc = this._rpc.bind(this);
      this.rpc = this.rpc.bind(this);
      this.client = client;
      this.options = options1;
      this.timeout = ((ref = this.options) != null ? ref.timeout : void 0) || DEFAULT_TIMEOUT;
      this.responses = new Cache({
        timeout: this.timeout
      });
      this.responses.on('expired', function(ev) {
        var ref1, ref2;
        if (typeof (ev != null ? (ref1 = ev.value) != null ? ref1.def.reject : void 0 : void 0) === 'function') {
          return ev.value.def.reject(new Error(`timeout: ${(ref2 = ev.value.options) != null ? ref2.info : void 0}`));
        }
      });
    }

    returnChannel() {
      if (!this._returnChannel) {
        this._returnChannel = this.client.queue('', {
          autoDelete: true,
          exclusive: true
        });
        this._returnChannel.then((q) => {
          return q.subscribe((msg, headers, deliveryInfo) => {
            return this.resolveResponse(deliveryInfo != null ? deliveryInfo.correlationId : void 0, msg, headers);
          });
        });
      }
      return this._returnChannel;
    }

    registerResponse(corrId, options = {}, deserialize = require('./error-deserializer')) {
      var def, value;
      def = {};
      def.promise = new Promise(function(resolve, reject) {
        def.resolve = resolve;
        return def.reject = reject;
      });
      value = {
        def: def,
        options: options,
        deserialize: deserialize
      };
      this.responses.set(corrId, value, options.timeout);
      return def;
    }

    resolveResponse(corrId, msg, headers) {
      var ct, p, response;
      if (response = this.responses.get(corrId)) {
        [ct, p] = decompress(msg, headers);
        return p.then((payload) => {
          this.responses.remove(corrId);
          return response.def.resolve(response.deserialize(payload));
        }).catch(function(err) {
          return response.def.reject(err);
        });
      }
    }

    _rpc(exchange, routingKey, msg, headers, options) {
      if (!msg) {
        throw new Error('Must provide msg');
      }
      return Promise.all([this.client.exchange(exchange), this.returnChannel()]).then(([ex, q]) => {
        var corrId, def, h, opts, p, timeout;
        // the correlation id used to match up request/response
        corrId = uuid.v4();
        // options stored locally
        options = options || {};
        options.info = options.info || `${ex.name}/${routingKey}`;
        // timeout
        timeout = options.timeout || this.timeout;
        // register the correlation id for response
        def = this.registerResponse(corrId, options);
        // options provided to server
        opts = {
          deliveryMode: 1,
          replyTo: q.name,
          correlationId: corrId,
          expiration: `${timeout}`
        };
        // construct headers
        opts.headers = headers || {};
        // the timeout is provided to server side so server can
        // discard queued up timed out requests.
        opts.headers.timeout = `${timeout}`;
        // maybe compress the payload
        [h, p] = compress(msg, options);
        merge(opts.headers, h);
        // wait for (maybe) compressed payload and then publish
        return p.then(function(payload) {
          return ex.publish(routingKey, payload, opts).then(function() {
            // promise for rpc return
            return def.promise;
          });
        });
      });
    }

    rpc(exchange, routingKey, msg, headers, options) {
      return this.client.compat.promise(this._rpc(exchange, routingKey, msg, headers, options));
    }

  };

}).call(this);
